@@ -4,7 +4,7 @@ import os
 import ruamel.yaml as yaml
 from typing import Any, Dict, List, Optional, Text, Union
 
-from rasa.shared.exceptions import RasaException
+from rasa.shared.exceptions import InvalidConfigException, RasaException
 import rasa.shared.utils.io
 import rasa.utils.io
 from rasa.shared.constants import (
@@ -17,13 +17,23 @@ from rasa.shared.utils.io import json_to_string
 logger = logging.getLogger(__name__)
 
 
-class InvalidConfigError(ValueError, RasaException):
-    """Raised if an invalid configuration is encountered."""
+# DEPRECATED: will be removed in Rasa Open Source 3.0
+InvalidConfigError = InvalidConfigException
 
 
 def load(
     config: Optional[Union[Text, Dict]] = None, **kwargs: Any
 ) -> "RasaNLUModelConfig":
+    """Create configuration from file or dict.
+
+    Args:
+        config: a file path, a dictionary with configuration keys. If set to
+            `None` the configuration will be loaded from the default file
+            path.
+
+    Returns:
+        Configuration object.
+    """
     if isinstance(config, Dict):
         return _load_from_dict(config, **kwargs)
 
@@ -96,14 +106,6 @@ class RasaNLUModelConfig:
         if self.__dict__["pipeline"] is None:
             # replaces None with empty list
             self.__dict__["pipeline"] = []
-        elif isinstance(self.__dict__["pipeline"], str):
-            # DEPRECATION EXCEPTION - remove in 2.1
-            raise RasaException(
-                f"You are using a pipeline template. All pipelines templates "
-                f"have been removed in 2.0. Please add "
-                f"the components you want to use directly to your configuration "
-                f"file. {DOCS_URL_MIGRATION_GUIDE}"
-            )
 
         for key, value in self.items():
             setattr(self, key, value)
