@@ -54,6 +54,7 @@ ACTIVE_LOOP = "active_loop"
 LOOP_INTERRUPTED = "is_interrupted"
 LOOP_REJECTED = "rejected"
 TRIGGER_MESSAGE = "trigger_message"
+FOLLOWUP_ACTION = "followup_action"
 
 # start of special user message section
 EXTERNAL_MESSAGE_PREFIX = "EXTERNAL: "
@@ -75,3 +76,5 @@ DEFAULT_KNOWLEDGE_BASE_ACTION = "action_query_knowledge_base"
 # represent the origin of a `SubState`
 USER = "user"
 SLOTS = "slots"
+
+ENTITY_LABEL_SEPARATOR = "#"
