@@ -1,6 +1,7 @@
 import copy
 import logging
 import os
+import time
 from collections import deque
 from enum import Enum
 from typing import (
@@ -17,10 +18,9 @@ from typing import (
     Union,
     FrozenSet,
     Tuple,
+    TYPE_CHECKING,
 )
 
-import typing
-
 import rasa.shared.utils.io
 from rasa.shared.constants import DEFAULT_SENDER_ID
 from rasa.shared.nlu.constants import (
@@ -31,7 +31,7 @@ from rasa.shared.nlu.constants import (
     ACTION_TEXT,
     ACTION_NAME,
 )
-from rasa.shared.core import events  # pytype: disable=pyi-error
+from rasa.shared.core import events
 from rasa.shared.core.constants import (
     ACTION_LISTEN_NAME,
     LOOP_NAME,
@@ -41,9 +41,11 @@ from rasa.shared.core.constants import (
     LOOP_REJECTED,
     TRIGGER_MESSAGE,
     LOOP_INTERRUPTED,
+    ACTION_SESSION_START_NAME,
+    FOLLOWUP_ACTION,
 )
-from rasa.shared.core.conversation import Dialogue  # pytype: disable=pyi-error
-from rasa.shared.core.events import (  # pytype: disable=pyi-error
+from rasa.shared.core.conversation import Dialogue
+from rasa.shared.core.events import (
     UserUttered,
     ActionExecuted,
     Event,
@@ -56,10 +58,10 @@ from rasa.shared.core.events import (  # pytype: disable=pyi-error
     SessionStarted,
     ActionExecutionRejected,
 )
-from rasa.shared.core.domain import Domain, State  # pytype: disable=pyi-error
+from rasa.shared.core.domain import Domain, State
 from rasa.shared.core.slots import Slot
 
-if typing.TYPE_CHECKING:
+if TYPE_CHECKING:
     from rasa.shared.core.training_data.structures import Story
     from rasa.shared.core.training_data.story_writer.story_writer import StoryWriter
 
@@ -121,8 +123,8 @@ class DialogueStateTracker:
 
         The dump should be an array of dumped events. When restoring
         the tracker, these events will be replayed to recreate the state."""
-
         evts = events.deserialise_events(events_as_dict)
+
         return cls.from_events(sender_id, evts, slots, max_event_history)
 
     @classmethod
@@ -130,13 +132,15 @@ class DialogueStateTracker:
         cls,
         sender_id: Text,
         evts: List[Event],
-        slots: Optional[List[Slot]] = None,
+        slots: Optional[Iterable[Slot]] = None,
         max_event_history: Optional[int] = None,
         sender_source: Optional[Text] = None,
     ):
         tracker = cls(sender_id, slots, max_event_history, sender_source)
+
         for e in evts:
             tracker.update(e)
+
         return tracker
 
     def __init__(
@@ -205,7 +209,7 @@ class DialogueStateTracker:
             "slots": self.current_slot_values(),
             "latest_message": self.latest_message.parse_data,
             "latest_event_time": latest_event_time,
-            "followup_action": self.followup_action,
+            FOLLOWUP_ACTION: self.followup_action,
             "paused": self.is_paused(),
             "events": _events,
             "latest_input_channel": self.get_latest_input_channel(),
@@ -381,9 +385,8 @@ class DialogueStateTracker:
 
     def init_copy(self) -> "DialogueStateTracker":
         """Creates a new state tracker with the same initial values."""
-
         return DialogueStateTracker(
-            DEFAULT_SENDER_ID,
+            self.sender_id or DEFAULT_SENDER_ID,
             self.slots.values(),
             self._max_event_history,
             is_rule_tracker=self.is_rule_tracker,
@@ -408,8 +411,11 @@ class DialogueStateTracker:
         yield tracker
 
     def applied_events(self) -> List[Event]:
-        """Returns all actions that should be applied - w/o reverted events."""
+        """Returns all actions that should be applied - w/o reverted events.
 
+        Returns:
+            The events applied to the tracker.
+        """
         loop_names = [
             event.name
             for event in self.events
@@ -584,10 +590,31 @@ class DialogueStateTracker:
             for e in domain.slots_for_entities(event.parse_data["entities"]):
                 self.update(e)
 
+    def update_with_events(
+        self,
+        new_events: List[Event],
+        domain: Optional[Domain],
+        override_timestamp: bool = True,
+    ) -> None:
+        """Adds multiple events to the tracker.
+
+        Args:
+            new_events: Events to apply.
+            domain: The current model's domain.
+            override_timestamp: If `True` refresh all timestamps of the events. As the
+                events are usually created at some earlier point, this makes sure that
+                all new events come after any current tracker events.
+        """
+        for e in new_events:
+            if override_timestamp:
+                e.timestamp = time.time()
+            self.update(e, domain)
+
     def as_story(self, include_source: bool = False) -> "Story":
         """Dump the tracker as a story in the Rasa Core story format.
 
-        Returns the dumped tracker as a string."""
+        Returns the dumped tracker as a string.
+        """
         from rasa.shared.core.training_data.structures import Story
 
         story_name = (
@@ -761,7 +788,6 @@ class DialogueStateTracker:
         ]
         return new_slots
 
-    # pytype: disable=bad-return-type
     @property
     def active_loop_name(self) -> Optional[Text]:
         """Get the name of the currently active loop.
@@ -773,8 +799,6 @@ class DialogueStateTracker:
 
         return self.active_loop.get(LOOP_NAME)
 
-    # pytype: enable=bad-return-type
-
     @property
     def latest_action_name(self) -> Optional[Text]:
         """Get the name of the previously executed action or text of e2e action.
@@ -815,3 +839,29 @@ def is_prev_action_listen_in_state(state: State) -> bool:
     """
     prev_action_name = state.get(PREVIOUS_ACTION, {}).get(ACTION_NAME)
     return prev_action_name == ACTION_LISTEN_NAME
+
+
+def get_trackers_for_conversation_sessions(
+    tracker: DialogueStateTracker,
+) -> List[DialogueStateTracker]:
+    """Generate trackers for `tracker` that are split by conversation sessions.
+
+    Args:
+        tracker: Instance of `DialogueStateTracker` to split.
+
+    Returns:
+        The trackers split by conversation sessions.
+    """
+    split_conversations = events.split_events(
+        tracker.events,
+        ActionExecuted,
+        {"action_name": ACTION_SESSION_START_NAME},
+        include_splitting_event=True,
+    )
+
+    return [
+        DialogueStateTracker.from_events(
+            tracker.sender_id, evts, tracker.slots, sender_source=tracker.sender_source
+        )
+        for evts in split_conversations
+    ]
