@@ -83,7 +83,7 @@ class RasaYAMLReader(TrainingDataReader):
         ):
             return TrainingData()
 
-        for key, value in yaml_content.items():  # pytype: disable=attribute-error
+        for key, value in yaml_content.items():
             if key == KEY_NLU:
                 self._parse_nlu(value)
             elif key == KEY_RESPONSES:
@@ -171,14 +171,12 @@ class RasaYAMLReader(TrainingDataReader):
         if isinstance(examples, list):
             example_tuples = [
                 (
-                    # pytype: disable=attribute-error
                     example.get(KEY_INTENT_TEXT, "").strip(STRIP_SYMBOLS),
                     example.get(KEY_METADATA),
                 )
                 for example in examples
                 if example
             ]
-        # pytype: enable=attribute-error
         elif isinstance(examples, str):
             example_tuples = [
                 (example, None)
