@@ -6,6 +6,7 @@ from collections import Counter, OrderedDict
 import copy
 from os.path import relpath
 from typing import Any, Dict, List, Optional, Set, Text, Tuple, Callable
+import operator
 
 import rasa.shared.data
 from rasa.shared.utils.common import lazy_property
@@ -60,17 +61,44 @@ class TrainingData:
 
         self._fill_response_phrases()
 
-    def merge(self, *others: "TrainingData") -> "TrainingData":
-        """Return merged instance of this data with other training data."""
+    def fingerprint(self) -> Text:
+        """Fingerprint the training data.
 
+        Returns:
+            hex string as a fingerprint of the training data.
+        """
+        relevant_attributes = {
+            "training_examples": list(
+                sorted(e.fingerprint() for e in self.training_examples)
+            ),
+            "entity_synonyms": self.entity_synonyms,
+            "regex_features": self.regex_features,
+            "lookup_tables": self.lookup_tables,
+            "responses": self.responses,
+        }
+
+        return rasa.shared.utils.io.deep_container_fingerprint(relevant_attributes)
+
+    def merge(self, *others: Optional["TrainingData"]) -> "TrainingData":
+        """Return merged instance of this data with other training data.
+
+        Args:
+            others: other training data instances to merge this one with
+
+        Returns:
+            Merged training data object. Merging is not done in place, this
+            will be a new instance.
+        """
         training_examples = copy.deepcopy(self.training_examples)
         entity_synonyms = self.entity_synonyms.copy()
         regex_features = copy.deepcopy(self.regex_features)
         lookup_tables = copy.deepcopy(self.lookup_tables)
         responses = copy.deepcopy(self.responses)
-        others = [other for other in others if other]
 
         for o in others:
+            if not o:
+                continue
+
             training_examples.extend(copy.deepcopy(o.training_examples))
             regex_features.extend(copy.deepcopy(o.regex_features))
             lookup_tables.extend(copy.deepcopy(o.lookup_tables))
@@ -108,10 +136,12 @@ class TrainingData:
         )
 
     def __hash__(self) -> int:
-        stringified = self.nlu_as_json() + self.nlg_as_markdown()
-        text_hash = rasa.shared.utils.io.get_text_hash(stringified)
+        """Calculate hash for the training data object.
 
-        return int(text_hash, 16)
+        Returns:
+            Hash of the training data object.
+        """
+        return int(self.fingerprint(), 16)
 
     @staticmethod
     def sanitize_examples(examples: List[Message]) -> List[Message]:
@@ -262,9 +292,7 @@ class TrainingData:
 
     def nlu_as_json(self, **kwargs: Any) -> Text:
         """Represent this set of training examples as json."""
-        from rasa.shared.nlu.training_data.formats import (  # pytype: disable=pyi-error
-            RasaWriter,
-        )
+        from rasa.shared.nlu.training_data.formats import RasaWriter
 
         return RasaWriter().dumps(self, **kwargs)
 
@@ -272,9 +300,7 @@ class TrainingData:
         """Generates the markdown representation of the response phrases (NLG) of
         TrainingData."""
 
-        from rasa.shared.nlu.training_data.formats import (  # pytype: disable=pyi-error
-            NLGMarkdownWriter,
-        )
+        from rasa.shared.nlu.training_data.formats import NLGMarkdownWriter
 
         return NLGMarkdownWriter().dumps(self)
 
@@ -284,9 +310,7 @@ class TrainingData:
         Returns:
             responses in yaml format as a string
         """
-        from rasa.shared.nlu.training_data.formats.rasa_yaml import (  # pytype: disable=pyi-error
-            RasaYAMLWriter,
-        )
+        from rasa.shared.nlu.training_data.formats.rasa_yaml import RasaYAMLWriter
 
         # only dump responses. at some point it might make sense to remove the
         # differentiation between dumping NLU and dumping responses. but we
@@ -295,16 +319,12 @@ class TrainingData:
 
     def nlu_as_markdown(self) -> Text:
         """Generates the markdown representation of the NLU part of TrainingData."""
-        from rasa.shared.nlu.training_data.formats import (  # pytype: disable=pyi-error
-            MarkdownWriter,
-        )
+        from rasa.shared.nlu.training_data.formats import MarkdownWriter
 
         return MarkdownWriter().dumps(self)
 
     def nlu_as_yaml(self) -> Text:
-        from rasa.shared.nlu.training_data.formats.rasa_yaml import (  # pytype: disable=pyi-error
-            RasaYAMLWriter,
-        )
+        from rasa.shared.nlu.training_data.formats.rasa_yaml import RasaYAMLWriter
 
         # avoid dumping NLG data (responses). this is a workaround until we
         # can remove the distinction between nlu & nlg when converting to a string
@@ -496,45 +516,109 @@ class TrainingData:
 
         Args:
             train_frac: percentage of examples to add to the training set.
-            random_seed: random seed
+            random_seed: random seed used to shuffle examples.
 
         Returns:
             Test and training examples.
         """
-        train, test = [], []
-        training_examples = set(self.training_examples)
 
-        def _split(_examples: List[Message], _count: int) -> None:
+        self.validate()
+
+        # Stratified split: both test and train should have (approximately) the
+        # same class distribution as the original data. We also require that
+        # each class is represented in both splits.
+
+        # First check that there is enough data to split at the requested
+        # rate: we must be able to include one example per class in both
+        # test and train, so num_classes is the minimum size of either.
+        smaller_split_frac = train_frac if train_frac < 0.5 else (1.0 - train_frac)
+        num_classes = (
+            len(self.number_of_examples_per_intent.items())
+            - len(self.retrieval_intents)
+            + len(self.number_of_examples_per_response)
+        )
+        num_examples = sum(self.number_of_examples_per_intent.values())
+
+        if int(smaller_split_frac * num_examples) + 1 < num_classes:
+            rasa.shared.utils.io.raise_warning(
+                f"There aren't enough intent examples in your data to include "
+                f"an example of each class in both test and train splits and "
+                f"also reserve {train_frac} of the data for training. "
+                f"The output training fraction will differ."
+            )
+
+        # Now simulate traversing the sorted examples, sampling at a rate
+        # of train_frac, so that after traversing k examples (for all k), we
+        # have sampled int(k * train_frac) of them for training.
+        # Corner case that makes this approximate: we require at least one sample
+        # in test, and at least one in train, so proportions will be less exact
+        # when classes have few examples, e.g. when a class has only 2 examples
+        # but the user requests an 80% / 20% split.
+
+        train, test = [], []
+
+        # helper to simulate the traversal of all examples in a single class
+        def _split_class(
+            _examples: List[Message], _running_count: int, _running_train_count: int
+        ) -> Tuple[int, int]:
             if random_seed is not None:
                 random.Random(random_seed).shuffle(_examples)
             else:
                 random.shuffle(_examples)
 
-            n_train = int(_count * train_frac)
-            train.extend(_examples[:n_train])
-            test.extend(_examples[n_train:])
+            # first determine how many samples we should have in training after
+            # traversing the examples in this class, if sampling train_frac of
+            # them. Then adjust so there's at least one example in test and train.
+            # Adjustment can accumulate until we encounter a frequent class.
+            exact_train_count = (
+                int((_running_count + len(_examples)) * train_frac)
+                - _running_train_count
+            )
+            approx_train_count = min(len(_examples) - 1, max(1, exact_train_count))
 
-        # to make sure we have at least one example per response and intent in the
-        # training/test data, we first go over the response examples and then go over
-        # intent examples
+            train.extend(_examples[:approx_train_count])
+            test.extend(_examples[approx_train_count:])
 
-        for response, count in self.number_of_examples_per_response.items():
+            return (
+                _running_count + len(_examples),
+                _running_train_count + approx_train_count,
+            )
+
+        training_examples = set(self.training_examples)
+        running_count = 0
+        running_train_count = 0
+
+        # Sort by class frequency so we first handle the tail of the distribution,
+        # where the percentages in the split are most approximate. Items from
+        # more frequent classes can then be over/ undersampled as needed to
+        # meet the requested train_frac. First for responses:
+        for response, _ in sorted(
+            self.number_of_examples_per_response.items(), key=operator.itemgetter(1)
+        ):
             examples = [
                 e
                 for e in training_examples
                 if e.get(INTENT_RESPONSE_KEY) and e.get(INTENT_RESPONSE_KEY) == response
             ]
-            _split(examples, count)
+            running_count, running_train_count = _split_class(
+                examples, running_count, running_train_count
+            )
             training_examples = training_examples - set(examples)
 
-        for intent, count in self.number_of_examples_per_intent.items():
+        # Again for intents:
+        for intent, _ in sorted(
+            self.number_of_examples_per_intent.items(), key=operator.itemgetter(1)
+        ):
             examples = [
                 e
                 for e in training_examples
                 if INTENT in e.data and e.data[INTENT] == intent
             ]
-            _split(examples, count)
-            training_examples = training_examples - set(examples)
+            if len(examples) > 0:  # will be 0 for retrieval intents
+                running_count, running_train_count = _split_class(
+                    examples, running_count, running_train_count
+                )
+                training_examples = training_examples - set(examples)
 
         return test, train
 
